import browser from "webextension-polyfill";
import log from "loglevel";
import axios from "axios";
import { getSettings } from "src/settings/settings";

let translationHistory = [];

const logDir = "common/translate";

const getHistory = (sourceWord, sourceLang, targetLang, translationApi) => {
  const history = translationHistory.find(
    history =>
      history.sourceWord == sourceWord &&
      history.sourceLang == sourceLang &&
      history.targetLang == targetLang &&
      history.translationApi == translationApi &&
      !history.result.isError
  );
  return history;
};

const setHistory = (sourceWord, sourceLang, targetLang, translationApi, result) => {
  translationHistory.push({
    sourceWord: sourceWord,
    sourceLang: sourceLang,
    targetLang: targetLang,
    translationApi: translationApi,
    result: result
  });
};

const sendRequestToGoogle = async (word, sourceLang, targetLang) => {
  const url = `https://translate.googleapis.com/translate_a/single?client=gtx&sl=${sourceLang}&tl=${targetLang}&dt=t&dt=bd&dj=1&q=${encodeURIComponent(
    word
  )}`;
  const result = await axios.get(url).catch(error => error.response);

  const resultData = {
    resultText: "",
    rawHTML: false,
    candidateText: "",
    sourceLanguage: "",
    percentage: 0,
    isError: false,
    errorMessage: ""
  };

  if (!result || result?.status !== 200) {
    resultData.isError = true;

    if (!result || result.status === 0) resultData.errorMessage = browser.i18n.getMessage("networkError");
    else if (result.status === 429 || result.status === 503) resultData.errorMessage = browser.i18n.getMessage("unavailableError");
    else resultData.errorMessage = `${browser.i18n.getMessage("unknownError")} [${result?.status} ${result?.statusText}]`;

    log.error(logDir, "sendRequest()", result);
    return resultData;
  }

  resultData.sourceLanguage = result.data.src;
  resultData.percentage = result.data.ld_result.srclangs_confidences[0];
  resultData.resultText = result.data.sentences.map(sentence => sentence.trans).join("");
  if (result.data.dict) {
    resultData.candidateText = result.data.dict
      .map(dict => `${dict.pos}${dict.pos != "" ? ": " : ""}${dict.terms !== undefined?dict.terms.join(", "):""}\n`)
      .join("");
  }

  log.log(logDir, "sendRequest()", resultData);
  return resultData;
};

const sendRequestToDeepL = async (word, sourceLang, targetLang) => {
  let params = new URLSearchParams();
  const authKey = getSettings("deeplAuthKey");
  params.append("auth_key", authKey);
  params.append("text", word);
  params.append("target_lang", targetLang);
  const url = getSettings("deeplPlan") === "deeplFree" ?
    "https://api-free.deepl.com/v2/translate" :
    "https://api.deepl.com/v2/translate";
  const result = await axios.post(url, params).catch(e => e.response);

  const resultData = {
    resultText: "",
    rawHTML: false,
    candidateText: "",
    sourceLanguage: "",
    percentage: 0,
    isError: false,
    errorMessage: ""
  };

  if (!result || result?.status !== 200) {
    resultData.isError = true;

    if (!result || result.status === 0) resultData.errorMessage = browser.i18n.getMessage("networkError");
    else if (result.status === 403) resultData.errorMessage = browser.i18n.getMessage("deeplAuthError");
    else resultData.errorMessage = `${browser.i18n.getMessage("unknownError")} [${result?.status} ${result?.statusText}] ${result?.data.message}`;

    log.error(logDir, "sendRequestToDeepL()", result);
    return resultData;
  }

  resultData.resultText = result.data.translations[0].text;
  resultData.sourceLanguage = result.data.translations[0].detected_source_language.toLowerCase();
  resultData.percentage = 1;

  log.log(logDir, "sendRequestToDeepL()", resultData);
  return resultData;
};

const sendRequestToSonaveeb = async (word) => {
  const resultData = {
    resultText: "",
    rawHTML: true,
    candidateText: "",
    sourceLanguage: "",
    percentage: 0,
    isError: false,
    errorMessage: ""
  };
  const domParser = new DOMParser();

  const mainPage = await _sendRequestToSonaveeb(`https://sonaveeb.ee/search/unif/dlall/dsall/${word}/1`, resultData);
  if (resultData.isError) {
    return resultData;
  }

  const mainDocument = domParser.parseFromString(mainPage.data, "text/html");
  const wordIdElement = mainDocument.querySelector("input[name=\"word-id\"]");
  if (wordIdElement) {
    const wordId = wordIdElement.getAttribute("value");

    const wordDetailsPage = await _sendRequestToSonaveeb(`https://sonaveeb.ee/worddetails/unif/${wordId}`, resultData);
    if (resultData.isError) {
      return resultData;
    }

    resultData.resultText = wordDetailsPage.data;

    const wordDetailsDocument = domParser.parseFromString(wordDetailsPage.data, "text/html");
    const morphoModalElement = wordDetailsDocument.querySelector("#morpho-modal-0");
    if (morphoModalElement) {
      const paradigmId = morphoModalElement.getAttribute("data-paradigm-id");

      const morphoPage = await _sendRequestToSonaveeb(`https://sonaveeb.ee/morpho/unif/${paradigmId}/est`, resultData);
      if (!resultData.isError) {
        resultData.resultText += morphoPage.data;
      } else {
        resultData.isError = false;
        resultData.errorMessage = "";
      }
    }
  } else {
    const wordFormElement = mainDocument.querySelector("a.word-form");
    if (!wordFormElement) {
      resultData.isError = true;
      resultData.errorMessage = `${browser.i18n.getMessage("sonaveebWordIdNotFound")}`;
      return resultData;
    }

    resultData.candidateText = wordFormElement.parentNode.innerHTML;
  }

  resultData.sourceLanguage = "et";
  resultData.percentage = 1;

  log.log(logDir, "sendRequestToSonaveeb()", resultData);
  return resultData;
};

const _sendRequestToSonaveeb = async (url, resultData) => {
  const result = await axios.get(url).catch(error => error.response);

  if (!result || result?.status !== 200) {
    resultData.isError = true;

    if (!result || result.status === 0) resultData.errorMessage = browser.i18n.getMessage("networkError");
    else resultData.errorMessage = `${browser.i18n.getMessage("unknownError")} [${result?.status} ${result?.statusText}] ${result?.data.message}`;

    log.error(logDir, "sendRequestToSonaveeb()", result);
  }

  return result;
};


export default async (sourceWord, sourceLang = "auto", targetLang, translationApi) => {
  log.log(logDir, "tranlate()", sourceWord, targetLang, translationApi);
  sourceWord = sourceWord.trim();
  if (sourceWord === "")
    return {
      resultText: "",
      candidateText: "",
      sourceLanguage: "en",
      percentage: 0,
      statusText: "OK"
    };

  const history = getHistory(sourceWord, sourceLang, targetLang);
  if (history) return history.result;

  const result = translationApi === "google"
    ? await sendRequestToGoogle(sourceWord, sourceLang, targetLang)
    : (translationApi === "deepl"
      ? await sendRequestToDeepL(sourceWord, sourceLang, targetLang)
      : await sendRequestToSonaveeb(sourceWord)
    );
  setHistory(sourceWord, sourceLang, targetLang, translationApi, result);
  return result;
};
